package com.example.polygonspheretest;

import java.util.Timer;
import java.util.TimerTask;

import android.content.Context;
import android.opengl.GLSurfaceView;
import android.view.MotionEvent;


public class MyGLView extends GLSurfaceView {

	MyRenderer myRenderer;

	public MyGLView(Context context) {
		super(context);
		myRenderer = new MyRenderer();
		setRenderer(myRenderer);
		setRenderMode(RENDERMODE_WHEN_DIRTY);  //without this, onDrawFrame() is called continuously and infinitely
	}

	float clickedX = 0, clickedY = 0;

	@Override public boolean onTouchEvent(MotionEvent e) {
		float x = e.getX();
		float y = e.getY();
		switch (e.getAction()) {
		case MotionEvent.ACTION_DOWN:
			System.out.println("ACTION_DOWN (" + x + "," + y + ")");
			clickedX = x;
			clickedY = y;
			break;
		case MotionEvent.ACTION_MOVE:
			float dx = x - clickedX;
			float dy = y - clickedY;
			System.out.println("ACTION_MOVE (" + x + "," + y + ")");
			if (dx != 0 || dy != 0) {
				MyQuat motion_from = new MyQuat();
				MyQuat motion_to = new MyQuat();
				float distance;
				int W = this.getWidth();
				int H = this.getHeight();

				distance = (float) Math.sqrt(dx * dx + dy * dy);

				motion_from.w = 0;
				motion_from.x = (float)clickedX / W - 0.5f;  /* normalized clicked_x */
				motion_from.y = 0.5f - (float)clickedY / H;  /* normalized clicked_y */
				motion_from.z = 1;  /* or 1-distance^2 */
				motion_from.normalize();

				motion_to.w = 0;
				motion_to.x = (float)x / W - 0.5f;
				motion_to.y = 0.5f - (float)y / H;
				motion_to.z = 1;  /* or 1-distance^2 */
				motion_to.normalize();

				myRenderer.draggedRot = motion_to.getRotFromQuats(motion_from, Math.atan(distance));
				myRenderer.draggedRot.normalize();  /* supress error */
			}
			requestRender();
			break;
		case MotionEvent.ACTION_UP:
			System.out.println("ACTION_UP (" + x + "," + y + ")");
			if (x == clickedX && y == clickedY){  /* no dragging */
				return true;
			}

			/* take dragged rotation in */
			myRenderer.currentRot = myRenderer.currentRot.mul(myRenderer.draggedRot);

			/* set angular velocity of invert rotation */
			myRenderer.springRot = myRenderer.draggedRot;
			myRenderer.springRot.scaleAngle(-0.2);
			/* reset to no drag rotation */
			myRenderer.draggedRot = new MyQuat();

			requestRender();

			Timer timer = new Timer(false);
			timer.schedule(new TimerTask() {
				public void run() {
					queueEvent(new Runnable() {
						// This method will be called on the rendering thread:
						public void run() {
							myRenderer.updateValuesByTick();
						}});
					if (myRenderer.springRot.x == 0 && myRenderer.springRot.y == 0 && myRenderer.springRot.z == 0) {
						setRenderMode(RENDERMODE_WHEN_DIRTY);
						cancel();
					}
				}
			}, 0, 30);

			setRenderMode(RENDERMODE_CONTINUOUSLY);

			break;
		}
		return true;
	}

}
