package com.example.polygonspheretest;

public class MyQuat {
	float w, x, y, z;

	public MyQuat() {
		w = 1;
		x = y = z = 0;
	}

	public MyQuat(MyQuat p) {
		w = p.w;
		x = p.x;
		y = p.y;
		z = p.z;
	}

	public MyQuat(float w, float x, float y, float z) {
		this.w = w;
		this.x = x;
		this.y = y;
		this.z = z;
	}

	void normalize()
	{
		double mag = Math.sqrt(w*w + x*x + y*y + z*z);
		w /= mag;
		x /= mag;
		y /= mag;
		z /= mag;
	}
	
	MyQuat getConjugate()
	{
		MyQuat tmp = new MyQuat();
		tmp.w =  w;
		tmp.x = -x;
		tmp.y = -y;
		tmp.z = -z;
		return tmp;
	}

	/* result.w <- p.w * q.w - p.v * q.v
	   result.v <- p.w * q.v + q.w * p.v + p.v × q.v
	   where v is vecter of (x,y,z) */
	MyQuat mul(MyQuat op)
	{
		MyQuat tmp = new MyQuat();
		tmp.w = w * op.w - (x * op.x + y * op.y + z * op.z);
		tmp.x = w * op.x + op.w * x + (y * op.z - z * op.y);
		tmp.y = w * op.y + op.w * y + (z * op.x - x * op.z);
		tmp.z = w * op.z + op.w * z + (x * op.y - y * op.x);
		return tmp;
	}

	/* rotator quat: q = cos(th/2) + v * sin(th/2) */
	MyQuat getRotation(float x, float y, float z, double angle)
	{
		MyQuat tmp = new MyQuat(this);
		double sin_angle = Math.sin(angle/2);

		tmp.w = 0;
		tmp.normalize();

		tmp.w = (float) Math.cos(angle/2);
		tmp.x *= sin_angle;
		tmp.y *= sin_angle;
		tmp.z *= sin_angle;
		return tmp;
	}

	/* rotator from 2 quat:
	   r.v <- p×q
	   r.w <- cos(phi/2)
	*/
	MyQuat crossProduct(MyQuat op)
	{
		MyQuat tmp = new MyQuat();
		tmp.w = 0;
		tmp.x = y * op.z - z * op.y;
		tmp.y = z * op.x - x * op.z;
		tmp.z = x * op.y - y * op.x;
		return tmp;
	}

	MyQuat getRotFromQuats(MyQuat op, double angle)
	{
		MyQuat cp = crossProduct(op);
		cp.w = (float) Math.cos(angle/2);
		return cp;
	}

	void scaleAngle(double scale)
	{
		double th = Math.acos(w) * 2;

		x /= Math.sin(th/2);
		y /= Math.sin(th/2);
		z /= Math.sin(th/2);

		if (scale < 0){
			x = -x;
			y = -y;
			z = -z;
			scale = -scale;
		}

		th *= scale;
		x *= Math.sin(th/2);
		y *= Math.sin(th/2);
		z *= Math.sin(th/2);
		w = (float) Math.cos(th/2);
	}

	void dampAngle(double attenuation)
	{
		double th = Math.acos(w) * 2;

		x /= Math.sin(th/2);
		y /= Math.sin(th/2);
		z /= Math.sin(th/2);
		
		th -= attenuation;
		if (th < 0.0){
			w = 1;
			x = y = z = 0;
		}else{
			x *= Math.sin(th/2);
			y *= Math.sin(th/2);
			z *= Math.sin(th/2);
			w = (float) Math.cos(th/2);
		}
	}

	/* convert quat to rotator matrix */
	double[] toMatrix()
	{
		double x2 = x * x;
		double y2 = y * y;
		double z2 = z * z;
		double xy = x * y;
		double yz = y * z;
		double zx = z * x;
		double wx = w * x;
		double wy = w * y;
		double wz = w * z;
		double m[] = new double[16];

		m[0] = 1 - 2 * (y2 + z2);  m[4] = 2 * (xy + wz);	  m[8] = 2 * (zx - wy);	   m[12] = 0;
		m[1] = 2 * (xy - wz);	  m[5] = 1 - 2 * (z2 + x2);  m[9] = 2 * (yz + wx);	   m[13] = 0;
		m[2] = 2 * (zx + wy);	  m[6] = 2 * (yz - wx);	  m[10] = 1 - 2 * (x2 + y2);  m[14] = 0;
		m[3] = 0;				  m[7] = 0;				  m[11] = 0;				  m[15] = 1;
		return m;
	}

	float[] toMatrixf()
	{
		double x2 = x * x;
		double y2 = y * y;
		double z2 = z * z;
		double xy = x * y;
		double yz = y * z;
		double zx = z * x;
		double wx = w * x;
		double wy = w * y;
		double wz = w * z;
		float m[] = new float[16];

		m[0] = (float) (1 - 2 * (y2 + z2));  m[4] = (float) (2 * (xy + wz));	  m[8] = (float) (2 * (zx - wy));	   m[12] = 0;
		m[1] = (float) (2 * (xy - wz));	  m[5] = (float) (1 - 2 * (z2 + x2));  m[9] = (float) (2 * (yz + wx));	   m[13] = 0;
		m[2] = (float) (2 * (zx + wy));	  m[6] = (float) (2 * (yz - wx));	  m[10] = (float) (1 - 2 * (x2 + y2));  m[14] = 0;
		m[3] = 0;				  m[7] = 0;				  m[11] = 0;				  m[15] = 1;
		return m;
	}

	/* Slerp of quaternion:
	                sin(a(1-t)) p + sin(a t) q
	    result(t) = --------------------------
	                          sin(a)
	   where cos(a) = p * q
	*/
	MyQuat slerp(MyQuat op, double t)
	{
		if (t == 0.0){
			return this;
		}
		else if (t == 1.0){
			return op;
		}
		else {
			MyQuat tmp = new MyQuat();
			double a;
			double sin_a_1_t, sin_at, sin_a;

			MyQuat np = new MyQuat(this), nq = new MyQuat(op);  /* normalized quaternion */
			np.w = 0;
			nq.w = 0;
			np.normalize();
			nq.normalize();
			a = Math.acos(np.x * nq.x + np.y * nq.y + np.z * nq.z);
			sin_a_1_t = Math.sin(a * (1-t));
			sin_at = Math.sin(a * t);
			sin_a = Math.sin(a);

			tmp.w = 0;
			tmp.x = (float) ((sin_a_1_t * np.x + sin_at * nq.x) / sin_a);
			tmp.y = (float) ((sin_a_1_t * np.y + sin_at * nq.y) / sin_a);
			tmp.z = (float) ((sin_a_1_t * np.z + sin_at * nq.z) / sin_a);
			return tmp;
		}
	}

}
