package com.example.polygonspheretest;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;

import javax.microedition.khronos.opengles.GL10;

public class MySphere {
	static final int granularity = 20;

	private final FloatBuffer mVertexBuffer;

	float vertex[];
	int[] texture;

	public MySphere(){
		int vn_per_one8th = (3 /*vertex*/ + 2 /*texCoord*/) * 3 /*triangle*/ * granularity * granularity;
		vertex = new float[vn_per_one8th * 8];

		addOne8thSurface( 1, 1, 1, vn_per_one8th * 0);
		addOne8thSurface( 1, 1,-1, vn_per_one8th * 1);
		addOne8thSurface( 1,-1, 1, vn_per_one8th * 2);
		addOne8thSurface( 1,-1,-1, vn_per_one8th * 3);
		addOne8thSurface(-1, 1, 1, vn_per_one8th * 4);
		addOne8thSurface(-1, 1,-1, vn_per_one8th * 5);
		addOne8thSurface(-1,-1, 1, vn_per_one8th * 6);
		addOne8thSurface(-1,-1,-1, vn_per_one8th * 7);

		ByteBuffer vbb = ByteBuffer.allocateDirect(vertex.length * 4);
		vbb.order(ByteOrder.nativeOrder());
		mVertexBuffer = vbb.asFloatBuffer();
		mVertexBuffer.put(vertex);
		mVertexBuffer.position(0);

	}

	final int TEX_WIDTH = 940;  /* = (300 * 3.14) */
	final int TEX_HEIGHT = 300;
	final int center1x =  50, center1y = 100;  /* center of red circle */
	final int center2x = 150, center2y = 150;  /* center of green circle */
	final int center3x = 250, center3y = 200;  /* center of blue circle */
	void addOne8thSurface(int xsign, int ysign, int zsign, int voffset)
	{
		MyQuat xunit = new MyQuat(0, xsign, 0, 0);
		MyQuat yunit = new MyQuat(0, 0, ysign, 0);
		MyQuat zunit = new MyQuat(0, 0, 0, zsign);
		MyQuat z1x, z1y, z2x, z2y;  /* start/end point on z1, z2 */
		MyQuat a, b, c, d;  /* vertices of a parallelogram */
		int z, t;
		int vn = voffset;  /* number of vertices */
		/* 	int nn = nnorm; */  /* number of normal vectors */

		if (xsign * ysign * zsign == -1){
			/* swap some 2 of 3 vector */
			yunit.z = zsign; yunit.y = 0;
			zunit.y = ysign; zunit.z = 0;
		}

		for(z = 1; z <= granularity; z++){
			z1x = zunit.slerp(xunit, (double)(z-1)/granularity);
			z1y = zunit.slerp(yunit, (double)(z-1)/granularity);
			z2x = zunit.slerp(xunit, (double)z/granularity);
			z2y = zunit.slerp(yunit, (double)z/granularity);

			/* triangle order by t:
			        a     c
			   z1x   ************  z1y
				    **  1 **  2 **
			       *  *  *  *  *  *
			      *t=0 **  1 **  2 *
			z2x  ********************  z2y
			            b     d    	       */

			/* leftmost(t=0) triangle */
			b = z2x;
			c = z1x;
			d = z2x.slerp(z2y, 1.0/z);
			vertex[vn++] = c.x;	vertex[vn++] = c.y;	vertex[vn++] = c.z;	//vertex(and maybe also normal)
			vertex[vn++] = texCoordU(xsign, ysign, zsign, c.x, c.y, c.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, c.x, c.y, c.z);

			vertex[vn++] = b.x;	vertex[vn++] = b.y;	vertex[vn++] = b.z;
			vertex[vn++] = texCoordU(xsign, ysign, zsign, b.x, b.y, b.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, b.x, b.y, b.z);

			vertex[vn++] = d.x;	vertex[vn++] = d.y;	vertex[vn++] = d.z;
			vertex[vn++] = texCoordU(xsign, ysign, zsign, d.x, d.y, d.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, d.x, d.y, d.z);

			for(t = 1; t < z; t++){
				a = z1x.slerp(z1y, z == 1 ? 0 : (double)(t-1)/(z-1));
				b = z2x.slerp(z2y, (double)t/z);
				c = z1x.slerp(z1y, z == 1 ? 0 : (double)t/(z-1));
				d = z2x.slerp(z2y, (double)(t+1)/z);

				vertex[vn++] = a.x;	vertex[vn++] = a.y;	vertex[vn++] = a.z;
				vertex[vn++] = texCoordU(xsign, ysign, zsign, a.x, a.y, a.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, a.x, a.y, a.z);

				vertex[vn++] = b.x;	vertex[vn++] = b.y;	vertex[vn++] = b.z;
				vertex[vn++] = texCoordU(xsign, ysign, zsign, b.x, b.y, b.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, b.x, b.y, b.z);

				vertex[vn++] = c.x;	vertex[vn++] = c.y;	vertex[vn++] = c.z;
				vertex[vn++] = texCoordU(xsign, ysign, zsign, c.x, c.y, c.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, c.x, c.y, c.z);


				vertex[vn++] = c.x;	vertex[vn++] = c.y;	vertex[vn++] = c.z;
				vertex[vn++] = texCoordU(xsign, ysign, zsign, c.x, c.y, c.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, c.x, c.y, c.z);

				vertex[vn++] = b.x;	vertex[vn++] = b.y;	vertex[vn++] = b.z;
				vertex[vn++] = texCoordU(xsign, ysign, zsign, b.x, b.y, b.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, b.x, b.y, b.z);

				vertex[vn++] = d.x;	vertex[vn++] = d.y;	vertex[vn++] = d.z;
				vertex[vn++] = texCoordU(xsign, ysign, zsign, d.x, d.y, d.z); vertex[vn++] = texCoordV(xsign, ysign, zsign, d.x, d.y, d.z);
			}
		}
	}
	private int dst2(int x, int y) {return x*x + y*y;}
	private boolean near(int x, int a, int b) {return a-b <= x && x <= a+b;}
	private float texCoordU(int xsign, int ysign, int zsign, float x, float y, float z){
		if (z == 0.0 && xsign == -1 && zsign == -1) {
			return 0.0f;
		} else {
			return (float) ((Math.atan2(z, x) / Math.PI + 1) / 2);
		}
	}
	private float texCoordV(int xsign, int ysign, int zsign, float x, float y, float z){
		return (y + 1) / 2;
	}

	public void draw(GL10 gl){
		float white[] = {0.7f, 0.7f, 0.7f, 1.0f};
		gl.glMaterialfv(GL10.GL_FRONT_AND_BACK, GL10.GL_DIFFUSE, white, 0);

		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		mVertexBuffer.position(0);
		gl.glVertexPointer(3, GL10.GL_FLOAT, (3+2)*4, mVertexBuffer);
		gl.glNormalPointer(GL10.GL_FLOAT, (3+2)*4, mVertexBuffer);
		gl.glEnableClientState(GL10.GL_NORMAL_ARRAY);
		mVertexBuffer.position(3);
		gl.glTexCoordPointer(2, GL10.GL_FLOAT, (3+2)*4, mVertexBuffer);
		gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
		gl.glEnable(GL10.GL_TEXTURE_2D);

		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		gl.glClearDepthf(1.0f);

		gl.glDrawArrays(GL10.GL_TRIANGLES, 0, 3 * granularity * granularity * 8);
		gl.glDisable(GL10.GL_TEXTURE_2D);
	}
	
	void initTexture(GL10 gl) {
		ByteBuffer bb = ByteBuffer.allocateDirect(TEX_WIDTH * TEX_HEIGHT * 4);
		bb.order(ByteOrder.BIG_ENDIAN);
		IntBuffer ib = bb.asIntBuffer();

		for (int y=0;y<TEX_HEIGHT;y++) {
			for (int x=0;x<TEX_WIDTH;x++) {
				boolean white = true;
				int red = 100;
				int green = 100;
				int blue = 100;

				/* circles */
				if (dst2(x%300 - center1x, y - center1y) < 1000){
					red = 255;  /* add red */
					white = false;
				}
				if (dst2(x%300 - center2x, y - center2y) < 1000){
					green = 255;  /* add green */
					white = false;
				}
				if (dst2(x%300 - center3x, y - center3y) < 1000){
					blue = 255;  /* add blue */
					white = false;
				}

				/* polar circles */
				if (y < 5 || y+5 > TEX_HEIGHT){
					red = 255;  /* add red */
					white = false;
				}

				/* sine curve */
				if (white && near(x%240 - 40, (int) (30 * Math.sin(2 * Math.PI * 5 * y / TEX_HEIGHT)), 3)){
					red = green = 255;  /* add yellow */
					white = false;
				}
				if (white && near(x%240 - 120, (int) (30 * Math.sin(2 * Math.PI * 5 * y / TEX_HEIGHT)), 3)){
					green = blue = 255;  /* add cyan */
					white = false;
				}
				if (white && near(x%240 - 200, (int) (30 * Math.sin(2 * Math.PI * 5 * y / TEX_HEIGHT)), 3)){
					blue = red = 255;  /* add magenta */
					white = false;
				}

				/* gray -> white */
				if (white){
					red = green = blue = 255;
				}

				// It seems like alpha is currently broken in Android...
				ib.put(red << 24 | green << 16 | blue << 8 | 0xFF);//255-alpha);
			}
		}
		ib.position(0);
		bb.position(0);

		texture = new int[1];
		gl.glGenTextures(1, texture, 0);
		gl.glBindTexture(GL10.GL_TEXTURE_2D, texture[0]);
		gl.glTexImage2D(GL10.GL_TEXTURE_2D, 0, GL10.GL_RGBA, TEX_WIDTH, TEX_HEIGHT, 0, GL10.GL_RGBA, GL10.GL_UNSIGNED_BYTE, bb);
		gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MIN_FILTER, GL10.GL_LINEAR);
		gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MAG_FILTER, GL10.GL_LINEAR);
	}
}
